from django.db import models
from django.test import TestCase, RequestFactory

from lemon import AdminSite

from .admin import PublicationAdmin
from .models import Publication


class Article(Publication):

    title = models.CharField(max_length=100)
    content = models.TextField()


class PublicationAdminTests(TestCase):

    def setUp(self):
        self.admin = PublicationAdmin(Article, AdminSite())
        self.request = RequestFactory().get('/')

    def get_list_display(self, *args, **kwargs):
        return self.admin.get_list_display(self.request, *args, **kwargs)

    def test_default_list_display(self):
        self.assertEqual(self.get_list_display(), (
            '__str__', 'author_name', 'publication_start_date', 'enabled',
        ))

    def test_custom_list_display(self):
        self.admin.list_display = ('title', 'content')
        self.assertEqual(self.get_list_display(), (
            'title', 'content', 'author_name', 'publication_start_date', 'enabled',
        ))

    def test_extend_list_display_is_disabled(self):
        self.admin.list_display = ('title', 'content')
        self.admin.extend_list_display = False
        self.assertEqual(self.get_list_display(), ('title', 'content'))

    def test_extend_list_display_is_disabled_by_argument(self):
        self.admin.list_display = ('title', 'content')
        self.assertEqual(self.get_list_display(extend=False), ('title', 'content'))



from django.db import models
from .querysets import PublicationQuerySet


class PublicationManager(models.Manager):

    def expired(self):
        return self.get_query_set().expired()

    def future(self):
        return self.get_query_set().future()

    def enabled(self):
        return self.get_query_set().enabled()

    def disabled(self):
        return self.get_query_set().disabled()

    def unpublished(self):
        return self.get_query_set().unpublished()

    def published(self):
        return self.get_query_set().published()

    def get_query_set(self):
        return PublicationQuerySet(self.model, using=self._db)



from django.db.models import Q
from django.db.models import query
from django.utils import timezone


class PublicationQuerySet(query.QuerySet):

    def expired(self):
        return self.filter(publication_end_date__lt=timezone.now())

    def future(self):
        return self.filter(publication_start_date__gt=timezone.now())

    def enabled(self):
        return self.filter(enabled=True)

    def disabled(self):
        return self.filter(enabled=False)

    def unpublished(self):
        return self.filter(~self._published_q())

    def published(self):
        return self.filter(self._published_q())

    def _published_q(self):
        now = timezone.now()
        q = Q(publication_end_date__gte=now)|Q(publication_end_date=None)
        q &= Q(publication_start_date__lte=now)
        q &= Q(enabled=True)
        return q



from django.views.generic import (
    ListView, DetailView, ArchiveIndexView, YearArchiveView, MonthArchiveView,
    WeekArchiveView, DayArchiveView, TodayArchiveView, DateDetailView,
)


class PublishedMixin(object):

    def get_queryset(self):
        queryset = super(PublishedMixin, self).get_queryset()
        return queryset.published()


class PublicationDateMixin(object):

    date_field = 'publication_start_date'


class PublicationListView(PublishedMixin, ListView):
    """Works like ListView but returns only published objects."""


class PublicationDetailView(PublishedMixin, DetailView):
    """Works like DetailView but returns object only if published."""


class PublicationArchiveIndexView(PublicationDateMixin, PublishedMixin,
                                  ArchiveIndexView):
    """Works like ArchiveIndexView but returns only published objects and uses
    publication_start_date as date_field.
    """


class PublicationYearArchiveView(PublicationDateMixin, PublishedMixin,
                                 YearArchiveView):
    """Works like YearArchiveView but returns only published objects and uses
    publication_start_date as date_field.
    """


class PublicationMonthArchiveView(PublicationDateMixin, PublishedMixin,
                                  MonthArchiveView):
    """Works like MonthArchiveView but returns only published objects and uses
    publication_start_date as date_field.
    """


class PublicationWeekArchiveView(PublicationDateMixin, PublishedMixin,
                                 WeekArchiveView):
    """Works like WeekArchiveView but returns only published objects and uses
    publication_start_date as date_field.
    """


class PublicationDayArchiveView(PublicationDateMixin, PublishedMixin,
                                DayArchiveView):
    """Works like DayArchiveView but returns only published objects and uses
    publication_start_date as date_field.
    """


class PublicationTodayArchiveView(PublicationDateMixin, PublishedMixin,
                                  TodayArchiveView):
    """Works like TodayArchiveView but returns only published objects and uses
    publication_start_date as date_field.
    """


class PublicationDateDetailView(PublicationDateMixin, PublishedMixin,
                                DateDetailView):
    """Works like DateDetailView but returns object only if published and uses
    publication_start_date as date_field.
    """



from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils.translation import ugettext_lazy as _


if 'lemon' in settings.INSTALLED_APPS:
    import lemon as admin
else:
    from django.contrib import admin


class PublicationAdmin(admin.ModelAdmin):
    """Default configuration for admin publications."""

    actions = ['make_enabled', 'make_disabled']
    fieldsets = (
        (_('Publication parameters'), {
            'classes': ('collapse', 'wide'),
            'fields': ('enabled',
                       'publication_start_date',
                       'publication_end_date')
        }),
    )
    date_hierarchy = 'publication_start_date'
    extend_list_display = True
    list_filter = ('enabled', 'publication_start_date')
    list_per_page = 25

    def get_author_admin_url(self, author):
        app_label = author._meta.app_label
        module_name = author._meta.module_name
        view_name = u'admin:{0}_{1}_change'.format(app_label, module_name)
        try:
            return reverse(view_name, args=(author.pk,))
        except NoReverseMatch:
            return ''

    def author_name(self, obj):
        """Show author full name and link to author's admin change page."""
        url = self.get_author_admin_url(obj.author)
        name = obj.author.get_full_name() or obj.author.get_username()
        if url:
            return u'<a href="%s">%s</a>' % (url, name)
        else:
            return name
    author_name.short_description = _('author name')
    author_name.allow_tags = True
    author_name.admin_order_field = 'author'

    def make_enabled(self, request, queryset):
        """Action which set enabled field to True."""
        queryset.update(enabled=True)
    make_enabled.short_description = _('Enable selected %(verbose_name_plural)s')

    def make_disabled(self, request, queryset):
        """Action which set enabled field to False."""
        queryset.update(enabled=False)
    make_disabled.short_description = _('Disable selected %(verbose_name_plural)s')

    def queryset(self, request):
        """Join author information to default queryset."""
        qs = super(PublicationAdmin, self).queryset(request)
        qs = qs.select_related('author')
        return qs

    def save_model(self, request, obj, form, change):
        """Set publication author from request."""
        if not change:
            obj.author = request.user
        super(PublicationAdmin, self).save_model(request, obj, form, change)

    def get_list_display(self, request, extend=None):
        if extend is None:
            extend = self.extend_list_display
        list_display = super(PublicationAdmin, self).get_list_display(request)
        if extend:
            list_display = tuple(list_display)
            list_display += ('author_name', 'publication_start_date', 'enabled')
        return list_display



from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from .managers import PublicationManager


USER_MODEL_PATH = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


class Publication(models.Model):
    """This abstract model provides publication-specific fields and tools."""

    author = models.ForeignKey(
        to=USER_MODEL_PATH,
        null=True,
        editable=False,
        verbose_name=_('author'),
        related_name='%(app_label)s_%(class)s_set',
    )
    enabled = models.BooleanField(
        verbose_name=_('enabled'),
        default=True,
        db_index=True,
        help_text=_('If not set, publication is hidden from visitors anyway.'),
    )
    publication_start_date = models.DateTimeField(
        verbose_name=_('publication start date'),
        default=timezone.now,
        db_index=True,
        help_text=_(
            'Publication will be visible to visitors starting from this date.'
        ),
    )
    publication_end_date = models.DateTimeField(
        verbose_name=_('publication end date'),
        null=True,
        blank=True,
        db_index=True,
        help_text=_(
            'Publication will be visible to visitors due to this date if set.'
        ),
    )

    objects = PublicationManager()

    class Meta:
        abstract = True
        ordering = ['-publication_start_date']



import os


TESTS_DIR = os.path.abspath(os.path.dirname(__file__))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    },
}

SITE_ID = 1

ROOT_URLCONF = 'tests.urls'

SECRET_KEY = 'secret key'

INSTALLED_APPS = (
    'lemon',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.admin',
    'publications',
)



import codecs
import os
from setuptools import setup, find_packages


def read(filename):
    filepath = os.path.join(os.path.dirname(__file__), filename)
    return codecs.open(filepath, encoding='utf-8').read()


setup(
    name='lemon-publications',
    version='0.2.1',
    license='ISC',
    description='Utils to create publications-like Django apps faster.',
    long_description=read('README.rst'),
    url='https://github.com/trilan/lemon-publications',
    author='Mike Yumatov',
    author_email='mike@yumatov.org',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



from django.contrib.syndication.views import Feed
from django.utils import feedgenerator


class PublicationFeed(Feed):

    model = None
    feed_type = feedgenerator.Rss201rev2Feed

    def items(self):
        return self.model.objects.published()[:10]

    def item_pubdate(self, item):
        return item.publication_start_date
